@@ -1,12 +1,12 @@
-# $Id: crossbinutils-1.0.tcl 69962 2010-07-24 00:36:58Z jmr@macports.org $
-# 
+# $Id: crossbinutils-1.0.tcl 100293 2012-12-07 14:36:29Z raimue@macports.org $
+#
 # Copyright (c) 2010 The MacPorts Project
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
 # met:
-# 
+#
 # 1. Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
 # 2. Redistributions in binary form must reproduce the above copyright
@@ -15,7 +15,7 @@
 # 3. Neither the name of The MacPorts Project nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -27,20 +27,20 @@
 # THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-# 
-# 
+#
+#
 # This PortGroup automatically sets all the fields of the various cross binutils
 # ports (e.g. spu-binutils).
-# 
+#
 # Usage:
-# 
+#
 #   PortGroup               crossbinutils 1.0
 #   crossbinutils.setup     spu 2.20.51.0.5
 
 options crossbinutils.target
 
 proc crossbinutils.setup {target version} {
-    global master_sites workpath worksrcpath
+    global master_sites workpath worksrcpath extract.suffix
 
     crossbinutils.target ${target}
 
@@ -48,16 +48,20 @@ proc crossbinutils.setup {target version} {
     version         ${version}
     categories      cross devel
     platforms       darwin
-    description     FSF Binutils for ${target} cross development
+    license         GPL-3+
+    maintainers     nomaintainer
 
+    description     FSF Binutils for ${target} cross development
     long_description \
         Free Software Foundation development toolchain ("binutils") for \
         ${target} cross development.
 
     homepage        http://www.gnu.org/software/binutils/binutils.html
-    master_sites    http://ftp.kernel.org/pub/linux/devel/binutils/
+    master_sites    gnu:binutils \
+                    http://mirrors.ibiblio.org/gnu/ftp/gnu/binutils/
     dist_subdir     binutils
     distname        binutils-${version}
+    worksrcdir      binutils-[string trimright ${version} {[a-zA-Z]}]
     use_bzip2       yes
 
     post-extract {
@@ -98,12 +102,23 @@ proc crossbinutils.setup {target version} {
                 ${worksrcpath}/${dir}/configure
         }
 
-        # Do not install libiberty
-        reinplace {/^install:/s/ .*//} ${worksrcpath}/libiberty/Makefile.in
+	# Install target-compatible libbfd/libiberty in the target's directory
+	reinplace "s|bfdlibdir=.*|bfdlibdir='${prefix}/${crossbinutils.target}/host/lib'|g" \
+		${worksrcpath}/bfd/configure                                \
+		${worksrcpath}/opcodes/configure
+	reinplace "s|bfdincludedir=.*|bfdincludedir='${prefix}/${crossbinutils.target}/host/include'|g"  \
+		${worksrcpath}/bfd/configure                                             \
+		${worksrcpath}/opcodes/configure
+
+	reinplace "s|\$(libdir)|\"${prefix}/${crossbinutils.target}/host/lib\"|g" \
+		${worksrcpath}/libiberty/Makefile.in
+	reinplace "s|\$(MULTIOSDIR)||g" \
+		${worksrcpath}/libiberty/Makefile.in
     }
 
     depends_lib \
-        port:gettext
+        port:gettext \
+        port:zlib
 
     configure.dir   ${workpath}/build
     configure.cmd   ${worksrcpath}/configure
@@ -113,7 +128,9 @@ proc crossbinutils.setup {target version} {
 
     configure.args \
         --target=${target} \
-        --program-prefix=${target}-
+        --program-prefix=${target}- \
+        --enable-install-libiberty \
+        --enable-install-libbfd
 
     build.dir ${workpath}/build
 
@@ -122,14 +139,14 @@ proc crossbinutils.setup {target version} {
     post-destroot {
         set docdir ${prefix}/share/doc/${name}
         xinstall -d ${destroot}${docdir}
-        xinstall -m 644 -W ${worksrcpath} COPYING COPYING.LIB COPYING.LIBGLOSS \
-            COPYING3 COPYING3.LIB ChangeLog MAINTAINERS README \
+        eval xinstall -m 644 \
+            [glob -type f ${worksrcpath}/{COPYING*,ChangeLog,MAINTAINERS,README*}] \
             ${destroot}${docdir}
     }
 
     universal_variant no
 
     livecheck.type  regex
-    livecheck.url   ${master_sites}
-    livecheck.regex {binutils-(\d+(?:\.\d+)*)}
+    livecheck.url   [lindex ${master_sites} 1]
+    livecheck.regex "binutils-((?!.*binutils.*|\\${extract.suffix}).*)\\${extract.suffix}"
 }
