@@ -1,8 +1,9 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# $Id: ruby-1.0.tcl 89756 2012-02-09 15:40:33Z ryandesign@macports.org $
-# 
-# Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
+# $Id: ruby-1.0.tcl 105953 2013-05-11 03:23:06Z ryandesign@macports.org $
+#
 # Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
+# Copyright (c) 2006-2013 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -17,7 +18,7 @@
 # 3. Neither the name of Apple Computer, Inc. nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 # "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 # LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -30,59 +31,125 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-# Define these variables assuming ruby1.8 to make them accessible in
-# the portfile after port group declaration. They can be modified by
-# ruby.setup, e.g. to use another ruby than 1.8.
-set ruby.bin            ${prefix}/bin/ruby
-set ruby.rdoc           ${prefix}/bin/rdoc
-set ruby.gem            ${prefix}/bin/gem
+# Usage:
+#
+#   1. use ruby.setup
+#
+#     PortGroup        ruby 1.0
+#     ruby.setup       module version type {} ruby19
+#
+#   2. use ruby.branch
+#
+#     PortGroup        ruby 1.0
+#     ruby.branch      1.9
+#     depends_lib      port:ruby${ruby.suffix}
+#     build.cmd        ${ruby.bin}
+
+# options:
+#   ruby.branch: select ruby version. 1.8, 1.9 or 2.0.
+#   ruby.link_binaries: whether generate suffixed symlink under ${prefix}/bin
+#        or not.
+# values:
+#   ruby.bin, ruby.rdoc, ruby.gem: fullpath to commands for ${ruby.branch}.
+#   ruby.suffix: suffix of portname. port:ruby${ruby.suffix} or
+#        port:rb${ruby.suffix}-foo.
+#   ruby.bindir: install location of commands without suffix from rb-foo.
+#   ruby.gemdir: install location of rubygems.
+#        such as "${prefix}/lib/ruby1.9/gems/1.9.1".
+#   ruby.link_binaries_suffix: suffix of commands from rb-foo under
+#        ${prefix}/bin. such as "-1.8" or "-1.9".
+#   (obsoleted values)
+#   ruby.prog_suffix: use ruby.branch.
+#   ruby.version: use ruby.api_version.
+# values from ruby.setup:
+#   ruby.module: port name without prefix. rb-${ruby.module}.
+#   ruby.project: project name at rubygems, rubyforge or sourceforge.
+
+options ruby.default_branch
+default ruby.default_branch 1.8
+options ruby.branch
+options ruby.bin ruby.rdoc ruby.gem ruby.bindir ruby.gemdir ruby.suffix
+option_proc ruby.branch ruby_set_branch
+proc ruby_set_branch {option action args} {
+    if {$action != "set"} {
+        return
+    }
+    global prefix ruby.branch \
+           ruby.bin ruby.rdoc ruby.gem ruby.bindir ruby.gemdir \
+           ruby.suffix ruby.link_binaries_suffix ruby.api_version \
+           ruby.prog_suffix
+    set ruby.bin            ${prefix}/bin/ruby${ruby.branch}
+    set ruby.rdoc           ${prefix}/bin/rdoc${ruby.branch}
+    set ruby.gem            ${prefix}/bin/gem${ruby.branch}
+    set ruby.bindir         ${prefix}/libexec/ruby${ruby.branch}
+    set ruby.gemdir         ${prefix}/lib/ruby${ruby.branch}/gems/${ruby.api_version}
+    # gem command for 1.8 from port:rb-rubygems
+    if {${ruby.branch} eq "1.8"} {
+        set ruby.gem        ${ruby.bindir}/gem
+        set ruby.gemdir     ${prefix}/lib/ruby/gems/${ruby.api_version}
+    }
+    set ruby.suffix         [join [split ${ruby.branch} .] {}]
+    if {${ruby.branch} eq "1.8"} {
+        set ruby.suffix     ""
+    }
+    set ruby.link_binaries_suffix -${ruby.branch}
+    set ruby.prog_suffix    ${ruby.branch}
+    if {${ruby.branch} eq "1.8"} {
+        set ruby.prog_suffix     ""
+    }
+}
 
 proc ruby.extract_config {var {default ""}} {
     global ruby.bin
-    if {[catch {set val [exec ${ruby.bin} -e "require 'rbconfig';puts Config::CONFIG\[\"${var}\"\]"]}]} {
+    if {[catch {set val [exec ${ruby.bin} -e "require 'rbconfig';puts RbConfig::CONFIG\[\"${var}\"\]"]}]} {
         set val ${default}
     }
     return $val
 }
 
-options ruby.version ruby.arch ruby.lib ruby.archlib
-default ruby.version    {[ruby.extract_config ruby_version]}
-default ruby.arch       {[ruby.extract_config arch "${os.arch}-${os.platform}${os.major}"]}
+options ruby.api_version ruby.lib ruby.archlib
+default ruby.api_version    {[ruby.extract_config ruby_version]}
+default ruby.arch           {[ruby.extract_config arch "${os.arch}-${os.platform}${os.major}"]}
 # define installation libraries as vendor location
-default ruby.lib        {[ruby.extract_config vendorlibdir ${prefix}/lib/ruby/vendor_ruby/${ruby.version}]}
-default ruby.archlib    {[ruby.extract_config vendorarchdir ${ruby.lib}/${ruby.arch}]}
+default ruby.lib            {[ruby.extract_config vendorlibdir ${prefix}/lib/ruby/vendor_ruby/${ruby.api_version}]}
+default ruby.archlib        {[ruby.extract_config vendorarchdir ${ruby.lib}/${ruby.arch}]}
+# ruby.version is obsoleted. use ruby.api_version.
+options ruby.version
+default ruby.version        {[ruby.extract_config ruby_version]}
 
-set ruby.module     ""
-set ruby.filename   ""
-set ruby.project    ""
-set ruby.docs       {}
-set ruby.srcdir     ""
+set ruby.module         ""
+set ruby.filename       ""
+set ruby.project        ""
+set ruby.docs           {}
+set ruby.srcdir         ""
+
+options ruby.link_binaries
+default ruby.link_binaries yes
+
+default ruby.branch         ${ruby.default_branch}
 
 # ruby group setup procedure; optional for ruby 1.8 if you want only
 # basic variables, like ruby.lib and ruby.archlib.
 proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {implementation "ruby"}} {
     global destroot prefix worksrcpath os.platform
     global ruby.bin ruby.rdoc ruby.gem
-    global ruby.version ruby.lib
+    global ruby.api_version ruby.lib ruby.suffix ruby.bindir ruby.gemdir
     global ruby.module ruby.filename ruby.project ruby.docs ruby.srcdir
+    global ruby.link_binaries_suffix
+    # ruby.version is obsoleted. use ruby.gemdir.
     global ruby.prog_suffix
-    
+
     if {${implementation} eq "ruby19"} {
-        set ruby.port_prefix rb19
-        set ruby.prog_suffix "1.9"
+        ruby.branch 1.9
+    } elseif {${implementation} eq "ruby20"} {
+        ruby.branch 2.0
     } elseif {${implementation} eq "ruby"} {
-        # ruby.bin, ruby.rdoc, and ruby.gem set to 1.8 by default
-        set ruby.port_prefix rb
-        # no program suffix by default, so leave as blank
-        set ruby.prog_suffix ""
+        ruby.branch 1.8
     } else {
-        ui_error "ruby.setup: unknown implementation '${implementation}' specified (ruby, ruby19 possible)"
+        ui_error "ruby.setup: unknown implementation '${implementation}' specified (ruby, ruby19, ruby20 possible)"
         return -code error "ruby.setup failed"
     }
-    set ruby.bin    ${prefix}/bin/ruby${ruby.prog_suffix}
-    set ruby.rdoc   ${prefix}/bin/rdoc${ruby.prog_suffix}
-    set ruby.gem    ${prefix}/bin/gem${ruby.prog_suffix}
-    
+
     # define ruby global names and lists
     # check if module is a list or string
     if {[llength ${module}] > 1} {
@@ -98,11 +165,11 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
         set ruby.filename   ${module}
     }
     set ruby.docs   ${docs}
-    
-    name            ${ruby.port_prefix}-[string tolower ${ruby.module}]
+
+    name            rb${ruby.suffix}-[string tolower ${ruby.module}]
     version         ${vers}
     categories      ruby
-    
+
     switch -glob ${source} {
         rubyforge:*:* {
             set num [lindex [split ${source} {:}] 1]
@@ -115,6 +182,7 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
         }
         rubyforge:* {
             set num [lindex [split ${source} {:}] 1]
+            set ruby.project [string tolower ${ruby.project}]
             homepage        http://rubyforge.org/projects/${ruby.project}
             master_sites    http://rubyforge.org/frs/download.php/${num}/
             livecheck.type  regex
@@ -122,7 +190,7 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             livecheck.regex "<strong>${ruby.project}</strong></td><td>(?:REL )?(.*)$"
         }
         rubyforge_gem:* {
-            set ruby.project [lindex [split ${source} {:}] 1]
+            set ruby.project [string tolower [lindex [split ${source} {:}] 1]]
             homepage        http://rubyforge.org/projects/${ruby.project}
             master_sites    http://gems.rubyforge.vm.bytemark.co.uk/gems/ \
                             http://rubyforge.iasi.roedu.net/gems/
@@ -131,6 +199,7 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             livecheck.regex "<strong>${ruby.module}</strong></td><td>(?:REL )?(.*)$"
         }
         rubyforge_gem {
+            set ruby.project [string tolower ${ruby.project}]
             homepage        http://rubyforge.org/projects/${ruby.project}
             master_sites    http://gems.rubyforge.vm.bytemark.co.uk/gems/ \
                             http://rubyforge.iasi.roedu.net/gems/
@@ -155,18 +224,18 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             master_sites    sourceforge:${ruby.project}
         }
     }
-    
+
     distname        ${ruby.filename}-${vers}
     dist_subdir     ruby
-    
+
     depends_lib     port:${implementation}
-    
+
     post-extract {
         # Create the work directory for gem-based ruby ports.
-        system "mkdir -p ${worksrcpath}"
+        file mkdir ${worksrcpath}
         system "find ${worksrcpath} -type d -name CVS | xargs rm -rf"
     }
-    
+
     switch -glob ${type} {
         basic_install.rb {
             post-patch {
@@ -187,11 +256,11 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
                 # adjust basic install.rb script
                 reinplace "s|site_ruby|vendor_ruby|" ${worksrcpath}/install.rb
             }
-            
+
             use_configure   no
-            
+
             build           {}
-            
+
             pre-destroot {
                 xinstall -d -m 0755 ${destroot}${ruby.lib}
             }
@@ -199,14 +268,19 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             destroot.cmd    ${ruby.bin} -rvendor-specific -rdestroot install.rb
             destroot.target
             destroot.destdir
+            post-destroot {
+                foreach file [readdir ${destroot}${prefix}/bin] {
+                    move [file join ${destroot}${prefix}/bin $file] ${destroot}${ruby.bindir}
+                }
+            }
         }
         copy_install:* {
             set ruby.srcdir [lindex [split ${type} {:}] 1]
-            
+
             use_configure   no
-            
+
             build           {}
-            
+
             destroot {
                 set root ${worksrcpath}/${ruby.srcdir}
                 xinstall -d -m 0755 ${destroot}${ruby.lib}
@@ -225,11 +299,11 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
         }
         install.rb {
             configure.cmd       ${ruby.bin} -rvendor-specific install.rb
-            configure.pre_args  config
-            
+            configure.pre_args  config --bin-dir=${destroot}${ruby.bindir}
+
             build.cmd           ${ruby.bin} -rvendor-specific install.rb
             build.target        setup
-            
+
             pre-destroot {
                 if {[file isfile ${worksrcpath}/config.save]} {
                     reinplace "s|^prefix=${prefix}|prefix=${destroot}${prefix}|g" \
@@ -247,10 +321,10 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
         setup.rb {
             configure.cmd       ${ruby.bin} -rvendor-specific setup.rb
             configure.pre_args  config
-            
+
             build.cmd           ${ruby.bin} -rvendor-specific setup.rb
             build.target        setup
-            
+
             pre-destroot {
                 if {[file isfile ${worksrcpath}/config.save]} {
                     reinplace "s|${prefix}|${destroot}${prefix}|g" \
@@ -264,19 +338,29 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             destroot.cmd        ${ruby.bin} -rvendor-specific setup.rb
             destroot.target     install
             destroot.destdir
+            post-destroot {
+                foreach file [readdir ${destroot}${prefix}/bin] {
+                    move [file join ${destroot}${prefix}/bin $file] ${destroot}${ruby.bindir}
+                }
+            }
         }
         extconf.rb {
-            configure.cmd       ${ruby.bin} -rvendor-specific extconf.rb
+            configure.cmd       ${ruby.bin} extconf.rb
             configure.pre_args
-            configure.args      --prefix=${prefix}
-            
-            build.args          RUBY="${ruby.bin} -rvendor-specific"
-            
-            destroot.args       RUBY="${ruby.bin} -rvendor-specific"
+            configure.args      --prefix=${prefix} --vendor
+
+            build.args          RUBY="${ruby.bin}"
+
+            destroot.args       RUBY="${ruby.bin}"
+            post-destroot {
+                foreach file [readdir ${destroot}${prefix}/bin] {
+                    move [file join ${destroot}${prefix}/bin $file] ${destroot}${ruby.bindir}
+                }
+            }
         }
         gnu {
             build.args          RUBY="${ruby.bin} -rvendor-specific"
-            
+
             pre-destroot {
                 if {[file isfile ${worksrcpath}/config.save]} {
                     reinplace "s|${prefix}|${destroot}${prefix}|g" \
@@ -292,25 +376,25 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
         gem {
             use_configure no
             extract.suffix .gem
-            
+
             if {${implementation} eq "ruby"} {
                 depends_lib-append  port:rb-rubygems
             }
-            
+
             extract {}
             build {}
-            
+
             pre-destroot {
-                xinstall -d -m 0755 ${destroot}${prefix}/lib/ruby${ruby.prog_suffix}/gems/${ruby.version}
+                xinstall -d -m 0755 ${destroot}${ruby.gemdir}
             }
-            
+
             destroot {
-                system "cd ${worksrcpath} && ${ruby.gem} install --local --force --install-dir ${destroot}${prefix}/lib/ruby${ruby.prog_suffix}/gems/${ruby.version} ${distpath}/${distname}"
-                
-                set binDir ${destroot}${prefix}/lib/ruby${ruby.prog_suffix}/gems/${ruby.version}/bin
+                system -W ${worksrcpath} "${ruby.gem} install --local --force --install-dir ${destroot}${ruby.gemdir} ${distpath}/${distname}.gem"
+
+                set binDir ${destroot}${ruby.gemdir}/bin
                 if {[file isdirectory $binDir]} {
                     foreach file [readdir $binDir] {
-                        file copy [file join $binDir $file] ${destroot}${prefix}/bin
+                        file copy [file join $binDir $file] ${destroot}${ruby.bindir}
                     }
                 }
             }
@@ -324,12 +408,23 @@ proc ruby.setup {module vers {type "install.rb"} {docs {}} {source "custom"} {im
             return -code error "ruby.setup failed"
         }
     }
-    
+
     if {$type != "gnu"} {
         configure.universal_args-delete  --disable-dependency-tracking
     }
-    
+
+    pre-destroot {
+        xinstall -d -m 0755 ${destroot}${ruby.bindir}
+    }
+
     post-destroot {
+        if {${ruby.link_binaries}} {
+            foreach bin [glob -nocomplain -tails -directory "${destroot}${ruby.bindir}" *] {
+                if {[catch {file type "${destroot}${prefix}/bin/${bin}${ruby.link_binaries_suffix}"}]} {
+                    ln -s "${ruby.bindir}/${bin}" "${destroot}${prefix}/bin/${bin}${ruby.link_binaries_suffix}"
+                }
+            }
+        }
         # Install documentation files (if specified)
         if {[llength ${ruby.docs}] > 0} {
             set docPath ${prefix}/share/doc/${name}
