@@ -1,12 +1,12 @@
-# $Id: texlive-1.0.tcl 77477 2011-04-01 00:36:31Z dports@macports.org $
-# 
+# $Id: texlive-1.0.tcl 107988 2013-07-10 04:20:53Z dports@macports.org $
+#
 # Copyright (c) 2010 Dan R. K. Ports <dports@macports.org>
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
 # met:
-# 
+#
 # 1. Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
 # 2. Redistributions in binary form must reproduce the above copyright
@@ -27,7 +27,7 @@
 # THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-# 
+#
 
 #
 # This portgroup contains common definitions for a MacPorts
@@ -47,13 +47,13 @@
 #
 
 # texmf files installed by texlive
-set texlive_texmfmain "${prefix}/share/texmf-texlive"
-
-# texmf files installed by texlive, but nominally distribution-independent
-# not clear it's really necessary to separate these from texmfmain,
-# but texlive goes to great effort to keep them separate, so we might as
-# well too
-set texlive_texmfdist "${prefix}/share/texmf-texlive-dist"
+# TeXLive used to maintain a separation between files that were
+# nominally distribution-independent ("texmf-dist") and those that
+# were tied to the binaries. This separation is now gone; everything
+# now goes into the texmfdist tree. $texlive_texmfmain points to the
+# same place for backwards-compatibility.
+set texlive_texmfdist "${prefix}/share/texmf-texlive"
+set texlive_texmfmain "${texlive_texmfdist}"
 
 # texmf files installed by ports other than texlive
 set texlive_texmfports "${prefix}/share/texmf"
@@ -67,6 +67,12 @@ set texlive_texmfsysvar "${prefix}/var/db/texmf"
 # configuration data from texconfig
 set texlive_texmfsysconfig "${prefix}/etc/texmf"
 
+# user's local texmf tree.
+# Note that this path is specified relative to the user's
+# home directory, i.e. it begins with an implicit ~/
+set texlive_texmfhome "Library/texmf"
+
+
 # location of binaries installed by texlive-bin
 #
 # All TeXLive binaries are built by texlive-bin, but most of them
@@ -84,7 +90,30 @@ set texlive_bindir "${prefix}/libexec/texlive/binaries"
 # This is provided to support MacTeX's TeX Distribution preference
 # pane: it can select the active TeX distribution by pointing the
 # /usr/texbin symlink here
+#
+# It also seems like the prefpane wants architecture-specific links,
+# and may want version-specific ones in the future, so create those in
+# ${texlive_mactex_texdistdir}.
 set texlive_mactex_texbindir "${prefix}/libexec/texlive/texbin"
+set texlive_mactex_texdistdir "${prefix}/libexec/texlive/texdist"
+
+# update texmf file path databases (ls-R)
+#
+# This should be run in the post-activate/deactivate hooks of any port
+# that installs texmf files. It updates the kpathsea database using
+# mktexlsr (formerly texhash), as well as ConTeXt's cache.
+proc texlive.mktexlsr {} {
+    global prefix
+
+    # Run mktexlsr. If that's not available, something's wrong.
+    system "${prefix}/bin/mktexlsr"
+
+    # If mtxrun is available (i.e. ConTeXt is installed), update its
+    # cache too. If it's not installed, that's OK.
+    if [file exists "${prefix}/bin/mtxrun"] {
+        system "${prefix}/bin/mtxrun --generate"
+    }
+}
 
 # Remove dependencies on any texlive-documentation-* ports, for use by
 # -doc variants
@@ -112,34 +141,62 @@ default texlive.formats {}
 default texlive.languages {}
 default texlive.maps {}
 
+# Whether to regenerate all config files, maps, and formats after
+# activation regardless of whether any new ones are installed.
 options texlive.forceupdatecnf
 default texlive.forceupdatecnf no
 
+# Whether to run mktexlsr after (de)activation. Usually required if
+# installing any texmf files.
+options texlive.use_mktexlsr
+default texlive.use_mktexlsr yes
+
+# Running mktexlsr after deactivation can be disabled separately. Note
+# that the value of this option is ignored if use_mktexlsr is disabled.
+options texlive.use_mktexlsr_on_deactivate
+default texlive.use_mktexlsr_on_deactivate yes
+
 proc texlive.texmfport {} {
     homepage        http://www.tug.org/texlive/
     platforms       darwin
-    
-    supported_archs noarch 
-    
-    master_sites    http://flute.csail.mit.edu/texlive/
+
+    supported_archs noarch
+    installs_libs   no
+
+    master_sites    http://giraffe.cs.washington.edu/texlive/
     use_xz          yes
 
-    global name master_sites
+    global name master_sites distname extract.suffix
     livecheck.type  regex
     livecheck.url   ${master_sites}
-    livecheck.regex ${name}-(\\d+)\\.tar
+    livecheck.regex ${name}-(\\d+)-run\\.tar
 
     depends_lib-append port:texlive-common port:texlive-bin
 
-    variant doc description "Install documentation" { }
-    variant src description "Install TeX source" { }
+    # distfile is split into three parts, all of which extract into
+    # $worksrcdir
+    # - $distname-run contains the runtime files required to install the
+    #   package, as well as the "tlpkginfo" directory containing metadata
+    # - $distname-doc contains optional documentation files
+    # - $distname-src contains optional source code for installed files
+    # The latter two are only downloaded if the corresponding variant
+    # is enabled. Currently, each package must have all three distfiles
+    # even if some are empty.
+    distfiles       ${distname}-run${extract.suffix}
+
+    variant doc description "Install documentation" {
+        distfiles-append ${distname}-doc${extract.suffix}
+    }
+    variant src description "Install TeX source" {
+        distfiles-append ${distname}-src${extract.suffix}
+    }
     default_variants +doc
 
     if {![variant_isset "doc"]} {
         # Skip any dependencies on texlive-documentation-* ports
         texlive.removedocdepends
     }
-    
+
     use_configure   no
 
     build           { }
@@ -152,7 +209,7 @@ proc texlive.texmfport {} {
         if {[variant_isset "src"]} { lappend indexlist "srcfiles" }
 
         # copy files listed in tlpkginfo/$indexname into destroot
-        foreach indexname $indexlist {            
+        foreach indexname $indexlist {
             set filelist [open ${worksrcpath}/tlpkginfo/${indexname}]
             while {[gets $filelist line] >= 0} {
                 # Check if file is excluded
@@ -172,7 +229,7 @@ proc texlive.texmfport {} {
                 if {$excluded} {
                     continue
                 }
-                    
+
                 set srcfile ${worksrcpath}/${indexname}/$line
 
                 # check for manpages and treat specially
@@ -202,7 +259,7 @@ proc texlive.texmfport {} {
                         default { ui_msg "warning: unknown file destination" }
                     }
                     set dstfile [join $splitline "/"]
-                    
+
                     # create directory if necessary, and install file
                     xinstall -d ${destroot}[file dirname $dstfile]
                     copy ${srcfile} ${destroot}${dstfile}
@@ -210,12 +267,6 @@ proc texlive.texmfport {} {
             }
         }
 
-        # create symlinks for any binaries activated by the port
-        foreach bin ${texlive.binaries} {
-            ln -s ${texlive_bindir}/$bin ${destroot}${prefix}/bin
-            ln -s ${texlive_bindir}/$bin ${destroot}${texlive_mactex_texbindir}
-        }
-
         # install a documentation file containing the list of TeX
         # packages installed. This also ensures that each port
         # provides at least one file, even if there's nothing to
@@ -256,24 +307,33 @@ proc texlive.texmfport {} {
                     "$fmtprefix$fmtname\t$fmtengine\t$fmtpatterns\t$fmtoptions"
 
                 # Simulate texlinks
-                if {![file exists ${destroot}${prefix}/bin/$fmtname]} {
+                if {[lsearch -exact ${texlive.binaries} $fmtname] != -1} {
                     # Decide what to link. Use the specified engine
                     # unless a binary with the same name as the
-                    # program exists (this can happen for metafont;
+                    # format exists (this can happen for metafont;
                     # see #28890)
-                    if {[file exists ${texlive_bindir}/$fmtname]} {
+                    #
+                    # It's OK if the binary named $fmtname is a broken
+                    # symlink, since we might be installing whatever
+                    # it's pointing to, hence the use of 'file lstat'.
+                    if {![catch {file lstat ${texlive_bindir}/$fmtname ignore}]} {
                         set linksource ${texlive_bindir}/$fmtname
                     } else {
                         set linksource ${prefix}/bin/$fmtengine
                     }
-                    
+
                     ln -s $linksource \
                         ${destroot}${prefix}/bin/$fmtname
                     ln -s $linksource \
                         ${destroot}${texlive_mactex_texbindir}/$fmtname
+
+                    # We've created the symlink for $fmtname; remove
+                    # it from texlive.binaries so we don't try to do
+                    # so again later.
+                    texlive.binaries-delete $fmtname
                 }
             }
-            
+
             close $fmtfile
         }
 
@@ -296,14 +356,21 @@ proc texlive.texmfport {} {
                 ${destroot}${texlive_texmfsysconfig}/language.d/10${name}.dat
             set langdeffilename \
                 ${destroot}${texlive_texmfsysconfig}/language.d/10${name}.def
+            set langluafilename \
+                ${destroot}${texlive_texmfsysconfig}/language.d/10${name}.dat.lua
             set langdatfile [open $langdatfilename "w"]
             set langdeffile [open $langdeffilename "w"]
+            set langluafile [open $langluafilename "w"]
+
             foreach x ${texlive.languages} {
                 set langname [lindex $x 0]
                 set langfile [lindex $x 1]
                 set langlhmin [lindex $x 2]
                 set langrhmin [lindex $x 3]
                 set langsyns [lindex $x 4]
+                set langpatt [lindex $x 5]
+                set langhyph [lindex $x 6]
+                set langspecial [lindex $x 7]
 
                 puts $langdatfile "$langname $langfile"
                 foreach syn $langsyns {
@@ -313,23 +380,68 @@ proc texlive.texmfport {} {
                 foreach syn [concat $langname $langsyns] {
                     puts $langdeffile "\\addlanguage{$syn}{$langfile}{}{$langlhmin}{$langrhmin}"
                 }
+
+                puts $langluafile "\t\['$langname'\] = {"
+                puts $langluafile "\t\tloader = '$langfile',"
+                puts $langluafile "\t\tlefthyphenmin = $langlhmin,"
+                puts $langluafile "\t\trighthyphenmin = $langrhmin,"
+                set qsyns {}
+                foreach syn $langsyns {
+                    lappend qsyns "'$syn'"
+                }
+                set qsynlist [join $qsyns ", "]
+                puts $langluafile "\t\tsynonyms = { $qsynlist },"
+                if {$langpatt != ""} {
+                    puts $langluafile "\t\tpatterns = '$langpatt',"
+                }
+                if {$langhyph != ""} {
+                    puts $langluafile "\t\thyphenation = '$langhyph',"
+                }
+                if {$langspecial != ""} {
+                    puts $langluafile "\t\tpatterns = '$langspecial',"
+                }
+                puts $langluafile "\t},\n"
             }
+
             close $langdatfile
             close $langdeffile
+            close $langluafile
+        }
+
+        # create symlinks for any binaries activated by the port
+        foreach bin ${texlive.binaries} {
+            ln -s ${texlive_bindir}/$bin ${destroot}${prefix}/bin
+            ln -s ${texlive_bindir}/$bin ${destroot}${texlive_mactex_texbindir}
         }
     }
 
     post-activate {
-        system "${texlive_bindir}/mktexlsr"
+        # Disable mktexlsr if it's not installed and this is a
+        # texlive-documentation-* port. It's possible (if unlikely) to
+        # install them without texlive-basic, so mktexlsr might not
+        # exist, but if it does we do want to run it.
+        if {[regexp {^texlive-documentation-} $name] &&
+            ![file exists ${prefix}/bin/mktexlsr]} {
+            texlive.use_mktexlsr no
+        }
+
+        if {${texlive.use_mktexlsr}} {
+            texlive.mktexlsr
+        }
+
         if {${texlive.forceupdatecnf}} {
             # If force was specified, update all the config files, and
             # regenerate all maps and formats.
             system "${prefix}/libexec/texlive-update-cnf language.dat"
             system "${prefix}/libexec/texlive-update-cnf language.def"
+            system "${prefix}/libexec/texlive-update-cnf language.dat.lua"
             system "${prefix}/libexec/texlive-update-cnf updmap.cfg"
             system "${prefix}/libexec/texlive-update-cnf fmtutil.cnf"
-            system "${prefix}/bin/updmap-sys"
-            system "${prefix}/bin/fmtutil-sys --all"
+            # map/format generation might fail if we are in the middle of
+            # a major upgrade and have not yet updated all texlive ports.
+            # Catch the error to prevent the upgrade from failing.
+            catch {system "${prefix}/bin/updmap-sys"}
+            catch {system "${prefix}/bin/fmtutil-sys --all"}
         } else {
             # Otherwise, only update the config files that are
             # actually affected, and only generate the needed
@@ -337,10 +449,11 @@ proc texlive.texmfport {} {
             if {${texlive.languages} != ""} {
                 system "${prefix}/libexec/texlive-update-cnf language.dat"
                 system "${prefix}/libexec/texlive-update-cnf language.def"
+                system "${prefix}/libexec/texlive-update-cnf language.dat.lua"
             }
             if {${texlive.maps} != ""} {
                 system "${prefix}/libexec/texlive-update-cnf updmap.cfg"
-                system "${prefix}/bin/updmap-sys"
+                catch {system "${prefix}/bin/updmap-sys"}
             }
             if {${texlive.formats} != ""} {
                 system "${prefix}/libexec/texlive-update-cnf fmtutil.cnf"
@@ -351,23 +464,33 @@ proc texlive.texmfport {} {
             # formats. Otherwise, just generate the formats we just
             # installed.
             if {${texlive.languages} != ""} {
-                system "${prefix}/bin/fmtutil-sys --all"                
+                catch {system "${prefix}/bin/fmtutil-sys --all"}
             } elseif {${texlive.formats} != ""} {
                 foreach x ${texlive.formats} {
                     set fmtname [lindex $x 1]
-                    system "${prefix}/bin/fmtutil-sys --byfmt $fmtname"
+                    catch {system "${prefix}/bin/fmtutil-sys --byfmt $fmtname"}
                 }
             }
         }
     }
 
     post-deactivate {
+        # Disable mktexlsr if it's not installed and this is a
+        # texlive-documentation-* port. See post-activate for why.
+        if {[regexp {^texlive-documentation-} $name] &&
+            ![file exists ${prefix}/bin/mktexlsr]} {
+            texlive.use_mktexlsr no
+        }
+
         # Update ls-R and any config files to reflect that the package
         # is now gone
-        system "${texlive_bindir}/mktexlsr"
+        if {${texlive.use_mktexlsr} && ${texlive.use_mktexlsr_on_deactivate}} {
+            texlive.mktexlsr
+        }
         if {${texlive.forceupdatecnf} || ${texlive.languages} != ""} {
             system "${prefix}/libexec/texlive-update-cnf language.dat"
             system "${prefix}/libexec/texlive-update-cnf language.def"
+            system "${prefix}/libexec/texlive-update-cnf language.dat.lua"
         }
         if {${texlive.forceupdatecnf} || ${texlive.maps} != ""} {
             system "${prefix}/libexec/texlive-update-cnf updmap.cfg"
@@ -375,7 +498,7 @@ proc texlive.texmfport {} {
         if {${texlive.forceupdatecnf} || ${texlive.formats} != ""} {
             system "${prefix}/libexec/texlive-update-cnf fmtutil.cnf"
         }
-        
+
         # Remove any generated format files
         foreach x ${texlive.formats} {
             set fmtname [lindex $x 1]
@@ -385,7 +508,7 @@ proc texlive.texmfport {} {
                 "mf-nowin" {set fmtengine "metafont"}
                 "mpost"    {set fmtengine "metapost"}
             }
-            
+
             foreach filename [glob -nocomplain ${texlive_texmfsysvar}/web2c/$fmtengine/$fmtname.*] {
                 delete $filename
             }
