@@ -14,7 +14,7 @@ $chemin="$DOCUMENT_ROOT/downloads";
 $rep=opendir($chemin);
 chdir($chemin);
 while($file = readdir($rep)) {
-	if($file != '..' && $file !='.' && (!eregi('php$', $file))) {
+	if($file != '..' && $file !='.' && (!preg_match('/\.php$/i', $file))) {
 		if(!is_dir($file)) {
 			echo "<a href=\"/downloads/$file\">$file</a>";
 			echo "<br>";
